using UnityEngine; //41 Post - Created by DimasTheDriver on Mar/04/2013 . Part of the 'Unity: expandable GUI Window' post. Available at: http://www.41post.com/?p=5118
using System.Collections;

public class ExpandableWindow : MonoBehaviour 
{
	//The rectangle that defines the current window position and area 
	private Rect windowRect;
	//The rectangle that holds the window values before it has been compacted / expanded
	public Rect oldWindowRect = new Rect(100, 20, 200, 200);
	
	//The window width and height when it's expanded 
	public int expandedWidth = 400;
	public int expandedHeight = 400;
	
	//Whether the window is expanded or compacted
	private bool expanded = false;
	
	//Whether the window is currently changing it's size
	private bool changingSize = false;
	
	//A float variable used for interpolation
	private float t = 0;
	
	void Start()
	{
		//Initially, 'windowRect' should have the same value as 'oldWindowRect'
		this.windowRect = this.oldWindowRect;
	}
	
    void OnGUI() 
	{
		//Render a window and save its position and size at 'windowRect'
        this.windowRect = GUI.Window(0, this.windowRect, this.WindowMethod, "Window Title");
    }
	
	//The window code
    void WindowMethod(int windowID) 
	{
		//If the window is expanded
		if(expanded)
		{
			//Render the 'Compact' button and define what happens when it's pressed: 
			if(GUI.Button(new Rect(this.windowRect.width - 110, this.windowRect.height - 40, 100, 30), "Compact") && !this.changingSize)
			{
				//Set 'expanded' to false
				this.expanded = false;
				//Start the 'Compact' Coroutine (defined below)
				StartCoroutine(Compact(0.03f));
			}
		}
		else //Window is compacted
		{
			//Render the 'Compact' button and define what happens when it's pressed: 
			if(GUI.Button(new Rect(this.windowRect.width - 110,this.windowRect.height - 40,100,30),"Expand") && !this.changingSize)
			{
				//Set 'true' to false
				this.expanded = true;
				//Start the 'Expand' Coroutine (defined below)
				StartCoroutine(Expand(0.03f));
			}
		}
		
		//If the window size isn't being changed
		if(!this.changingSize)
		{
			//Make the entire window background to act as a drag area	
        	GUI.DragWindow();
		}
    }
	
	//The coroutine for expanding the window
	private IEnumerator Expand(float rate)
	{
		//Update the 'oldWindowRect' with the current window position
		this.oldWindowRect.x = this.windowRect.x;
		this.oldWindowRect.y = this.windowRect.y;
		
		//Calculate the new window 'x' position.
		float newXpos = this.oldWindowRect.x - (this.expandedWidth - this.oldWindowRect.width)/2 ;
		float newYpos = this.oldWindowRect.y - (this.expandedHeight - this.oldWindowRect.height)/2 ;
		
		//Set 'changingSize' to true, so that the window can't be dragged while it's being expanded
		this.changingSize = true;
		
		//If 't' is smaller than one
		while (this.t < 1.0f)
		{
			//Increment the variable 't' based on 'rate' passed as an argument
			this.t += rate;
		
			/*Expand the window by interpolating the width, height, x and y positions to their new
			 * values*/
			this.windowRect.width = Mathf.SmoothStep(this.oldWindowRect.width, this.expandedWidth, t);
			this.windowRect.x = Mathf.SmoothStep(this.oldWindowRect.x, newXpos, t);
			
			this.windowRect.height = Mathf.SmoothStep(this.oldWindowRect.height, this.expandedHeight, t);
			this.windowRect.y = Mathf.SmoothStep(this.oldWindowRect.y, newYpos, t);
			
			//Yield (wait) the code execution for the time defined at 'rate'
			yield return new WaitForSeconds(rate); 
		}
		
		//Set 'changingSize' to false. This makes the window draggable again.
		this.changingSize = false;
	}
	
	//The coroutine for compacting the window
	private IEnumerator Compact(float rate)
	{
		//Update the 'oldWindowRect' with the current window position
		this.oldWindowRect.x = this.windowRect.x;
		this.oldWindowRect.y = this.windowRect.y;
		
		//Calculate the new window position.
		float newXpos = this.oldWindowRect.x + (this.expandedWidth - this.oldWindowRect.width)/2 ;
		float newYpos = this.oldWindowRect.y + (this.expandedHeight - this.oldWindowRect.height)/2 ;
		
		//Set 'changingSize' to true, so that the window can't be dragged while it's being compacted
		this.changingSize = true;
		
		while (this.t > 0.0f)
		{
			//Decrement the variable 't' based on 'rate' passed as an argument
			this.t -= rate;
	
			/*Compact the window by interpolating the width, height, x and y positions to their new
			 * values*/
			this.windowRect.width = Mathf.SmoothStep(this.oldWindowRect.width, this.expandedWidth, t);
			this.windowRect.x = Mathf.SmoothStep(newXpos, this.oldWindowRect.x, t);
			
			this.windowRect.height = Mathf.SmoothStep(this.oldWindowRect.height, this.expandedHeight, t);
			this.windowRect.y = Mathf.SmoothStep(newYpos, this.oldWindowRect.y, t);
			
			//Yield (wait) the code execution for the time defined at 'rate'
			yield return new WaitForSeconds(rate); 
		}
		
		//Set 'changingSize' to true, so that the window can't be dragged while it's being compacted
		this.changingSize = false;
	}
}
